from boto.s3.key import Key
from boto.s3.connection import S3Connection
import os
import boto3

class S3Store(object):
    def __init__(self,key_id,key_secret,host,port,bucket_name):
        self.conn = S3Connection(
            aws_access_key_id = key_id,
            aws_secret_access_key = key_secret,
            host = host,
            port = port,
            is_secure = False,
            calling_format='boto.s3.connection.OrdinaryCallingFormat'
        )
        self.bucket_name = bucket_name
        try:
            self.bucket = self.conn.get_bucket(self.bucket_name)
        except:
            pass
            #self.bucket = self.conn.create_bucket(self.bucket_name)
    
    def upload(self,local_path,remote_path):
        package_name = os.path.basename(local_path)
        package_key = Key(self.bucket, remote_path)
        packege_key.set_contents_from_filename(local_path)
        return True
    
    def download(self,remote_path,local_path):
        package_key = Key(self.bucket, remote_path)
        package_key.get_contents_to_filename(local_path)

    def dir(self,opath):
        rt = []
        path = ''.strip()
        if (path.startswith("/")):
            path = path[1:]
        for k in self.bucket.list(prefix = path,delimiter = "/"):
            rt.append(k.name)
        return rt
    
    def rename(self,old_name,new_name):
        package_old_key = Key(self.bucket, old_name)
        package_new_key = Key(self.bucket, new_name)
        if package_old_key.exists() and (not package_new_key.exists()):
            package_old_key.copy(self.bucket, package_new_key)
        if package_new_key.exists():
            package_old_key.delete()
        return True
    
    def delete_packetage(self, package_name):
        package_key = Key(self.bucket, package_name)
        if package_key.exists():
            package_key.delete()
        else:
            raise ValueError('package:%s are not exist' % package_name)
        return True


#coding=utf-8
from __future__ import division
from __future__ import print_function
import time
from collections import OrderedDict
import subprocess

class hoster:
    #host =  user:passwd@127.0.0.1:22
    def __init__(self,host = None):
        self.hostinfo = host
        self.ssh = None
        self.cpu_stat_head = ['user','nice','system','idle','iowait','irq','softirq','stealstolen','guest']
        return
    def __del__(self):
        if (self.hostinfo != None and self.ssh != None):
            self.ssh.close()

    def __connect(self):
        if (self.hostinfo != None and self.ssh == None):
            self.port = 22
            self.user,self.host = tuple(self.hostinfo.split('@'))
            self.user,self.passwd = tuple(self.user.split(':'))
            if (len(self.host.split(':')) == 2):
                t = self.host.split(':')
                self.host = t[0]
                self.port = int(t[1])

            import paramiko
            self.ssh = paramiko.SSHClient()
            self.ssh.set_missing_host_key_policy(paramiko.AutoAddPolicy())
            self.ssh.connect(self.host,self.port,self.user,self.passwd)
            #self.sftp_client = self.ssh.open_sftp()          

    def __get_out(self,cmd):
        self.__connect()
        ret = []
        if (self.ssh != None):
            stdin, stdout, stderr = self.ssh.exec_command('%s' %cmd)
            while True:
                line = stdout.readline()
                if len(line) <= 0:
                    break
                ret.append(line)
        else:
            p = subprocess.Popen(cmd.split(' '), stdout = subprocess.PIPE)
            data = p.stdout.read()
            ret = data.split("\n")
            if (ret[-1] == ''):
                ret.pop()
        return ret       

    def __cpu_one(self):
        ret = []
        #user nice system idle iowait irq softirq stealstolen guest
        ctx = self.__get_out('cat /proc/stat')
        for line in ctx:
            l = line.split()
            if len(l) < 5:
                continue
            if (l[0].startswith('cpu')):
                t = [l[0]]
                t.extend([int(x) for x in l[1:]])
                ret.append(t)
        return ret
    #cpu使用百分比
    def cpu(self):
        info1 = self.__cpu_one()[0]
        print(info1)
        tot1 = info1[1] + info1[2] + info1[3] + info1[4] + info1[5] + info1[6] + info1[7] + info1[8]
        use1 = info1[1] + info1[2] + info1[3]
        time.sleep(2)
        info2 = self.__cpu_one()[0]
        tot2 = info2[1] + info2[2] + info2[3] + info2[4] + info2[5] + info2[6] + info2[7] + info2[8]
        use2 = info2[1] + info2[2] + info2[3]
        return round((use2-use1) * 1.0/(tot2-tot1) * 100,2)
    #系统负载1,5,15分钟
    def loadavg(self):
        ctx = self.__get_out('cat /proc/loadavg')
        ret = {}
        if (len(ctx) > 0):
            avg = ctx[0].split()
            ret['1'] = float(avg[0])
            ret['5'] = float(avg[1])
            ret['15'] = float(avg[2])
        return ret

    def mem(self):
        meminfo = {}
        #user nice system idle iowait irq softirq stealstolen guest
        ctx = self.__get_out('cat /proc/meminfo')
        for line in ctx:
            if (len(line.split(':')) == 2):
                v = line.split(':')[1].strip().split(' ')
                if (len(v) == 2 and v[1] == 'kB'):
                     meminfo[line.split(':')[0]] = int(v[0]) * 1024
                elif (len(v) == 2 and v[1] == 'mB'):
                    meminfo[line.split(':')[0]] = int(v[0]) * 1024 * 1024
                else:
                    meminfo[line.split(':')[0]] = int(v[0])
        return meminfo
    #网卡数据
    def net(self,interface = 'eth0'):
        ret = []
        ctx = self.__get_out('cat /proc/net/dev')
        for inter in  ctx:
            if interface in inter:
                stat = float(inter.split()[1])
                ret[0:] = [stat]
                stat = float(inter.split()[9])
                ret[1:] = [stat]
        return ret

    #磁盘
    def disk(self):
        ret = {}
        ctx = self.__get_out('df -h')
        cnt = 0
        for line in ctx:
            cnt = cnt + 1
            if (cnt == 1):
                continue
            l = line.split()
            if len(l) == 0 or (len(l) > 1 and l[1] == 'Size'):
                continue
            ret[l[0]] = l
        return ret

    #文件节点
    def dfi(self):
        dfi = {}
        ctx = self.__get_out('df -i')
        cnt = 0
        for line in ctx:
            cnt = cnt + 1
            if (cnt == 1):
                continue
            l = line.split()
            if len(l) == 0 or (len(l) > 1 and l[1] == 'Size'):
                continue
            dfi[l[5]] = l[4]
        #print dfi
        return dfi
    


    #cpu处理器型号
    def cputype(self):
        ctx = self.__get_out('cat /proc/cpuinfo')
        nprocs = 0
        cpuinfo = {}
        procinfo = {}
        for line in ctx:
            if not line.strip():
                cpuinfo['proc%s' % nprocs] = procinfo
                nprocs = nprocs + 1
                procinfo = {}
            else:
                if len(line.split(':')) == 2:
                    procinfo[line.split(':')[0].strip()] = line.split(':')[1].strip()
                else:
                    procinfo[line.split(':')[0].strip()] = '' 
        return cpuinfo
    
    
if __name__ == '__main__':
    h = hoster('appuser:MaxusTest2020_@10.130.10.43:22')
    print(h.cputype())
    print(h.cpu())
    print(h.mem())
    print(h.disk())
    print(h.loadavg())
    print(h.dfi())






import datetime
import time


def utc2local(utc_st,fmt = None):
    '''UTC时间转本地时间（+8:00)'''
    now_stamp = time.time()
    local_time = datetime.datetime.fromtimestamp(now_stamp)
    utc_time = datetime.datetime.utcfromtimestamp(now_stamp)
    offset = local_time - utc_time
    if (type(utc_st) == str):
        local_st = datetime.datetime.strptime(utc_st, '%Y-%m-%dT%H:%M:%S%fZ') + offset
    else:
        local_st = utc_st + offset
    if (fmt != None):
        return local_st.strftime(fmt)
    else:
        return local_st


def local2utc(local_st):
    '''本地时间转UTC时间（-8:00）'''
    time_struct = time.mktime(local_st.timetuple())
    utc_st = datetime.datetime.utcfromtimestamp(time_struct)
    return utc_st    

'''
    end_tm 默认为当前时间
    from_tm 可以时间戳1541038591 或者 '2018-11-01 10:26:30'，'2018-11-01T00:09:36Z'
    from_fmt 只有当from_tm为字符时候才生效，默认是'%Y-%m-%dT%H:%M:%S%fZ'，例如：'%Y-%m-%d %H:%M:%S'
    end_fmt 同上
'''
def diff(from_tm,end_tm = None,from_fmt = None,end_fmt = None):
    from_dt = None
    utc_flag = False
    if type(from_tm) == float or type(from_tm) == int:
        from_dt = datetime.datetime.fromtimestamp(from_tm)
    elif type(from_tm) == str:
        if (from_fmt != None):
            from_dt = datetime.datetime.strptime(from_tm,from_fmt)
        else:
            from_dt = datetime.datetime.strptime(from_tm,'%Y-%m-%dT%H:%M:%S%fZ')
            utc_flag = True
    end_dt = None
    if (end_tm != None):
        if type(end_tm) == float or type(end_tm) == int:
            end_dt = datetime.datetime.fromtimestamp(end_tm)
        elif type(end_tm) == str:
            if (end_fmt != None):
                end_dt = datetime.datetime.strptime(end_tm,end_fmt)
            else:
                end_dt = datetime.datetime.strptime(end_tm,'%Y-%m-%dT%H:%M:%S%fZ')
    else:
        end_dt = datetime.datetime.utcfromtimestamp(time.time())
    return (end_dt - from_dt).seconds



if __name__ == '__main__':
    l_stp = time.time()
    l_tm = datetime.datetime.fromtimestamp(l_stp)
    print(local2utc(l_tm))
    utc_dt = datetime.datetime.strptime('2018-11-01T00:09:36Z', '%Y-%m-%dT%H:%M:%S%fZ')
    print(utc2local(utc_dt))
    print(utc2local('2018-11-01T00:09:36Z'))
    print(utc2local(utc_dt,'%Y-%m-%d %H:%M:%S'))
    print(diff('2018-11-01T04:00:46Z'))
    print(diff('2018-11-01 10:26:30',None,'%Y-%m-%d %H:%M:%S'))






from kazoo.client import KazooClient
from kazoo.client import KazooState
import functools
import json

class ZKUtil(object):
    def __init__(self,hosts):
        self._hosts = hosts
        self.client = None

    def connect_required(func):
        @functools.wraps(func)
        def handle_func(*args, **kwargs):
            self = args[0]
            if (not self.client):
                self.connect()
            rt = func(*args, **kwargs)
            return rt
        return handle_func    

    def connect(self):
        self.client = KazooClient(self._hosts, timeout = 10)
        self.client.start()

    @connect_required
    def get(self,path):
        return self.client.get(path)
    
    @connect_required
    def children(self,path):
        return self.client.get_children(path)


if __name__ == '__main__':
    z = ZKUtil('10.130.9.127:2181')
    t = z.children('/activemq/leveldb-stores')
    print(t)
    for k in t:
        m = z.get('/activemq/leveldb-stores/' + k)
        dt = json.loads(m[0].decode())
        print("address:{},elected:{}".format(dt["address"],dt["elected"]))




import stomp
import functools
import time
from .zk_util import ZKUtil
import json
import logging

class AMQListener(object):
    def __init__(self,fun):
        self._fun = fun
    def on_message(self, headers, message):
        self._fun(headers,message)

    def on_error(self, headers, message):
        print(headers)
        print(message)

class AMQUtil(object):
    def __init__(self,ip,port,user,passwd):
        self._ip = ip
        self._port = port
        self._user = user
        self._passwd = passwd
        self.client = None
    
    def connect_required(func):
        @functools.wraps(func)
        def handle_func(*args, **kwargs):
            self = args[0]
            if (not self.client):
                self.connect()
            rt = func(*args, **kwargs)
            return rt
        return handle_func

    def connect(self):
        self.client = stomp.Connection10([(self._ip,self._port)])
        self.client.start()
        self.client.connect(login=self._user, passcode=self._passwd)
    
    @connect_required
    def send2queue(self,qname,msg):
        self.client.send(qname,msg)

    @connect_required
    def send2topic(self,ttopic,msg):
        self.client.send(ttopic,msg)

    @connect_required
    def recvmsg(self,qname,fun):
        self.client.set_listener('Listener', AMQListener(fun))
        self.client.subscribe(qname, ack='auto', transformation="jms-json")

    @connect_required
    def check(self,master):
        self._flag = True
        def fun(header,msg):
            logging.info("header:%s,msg:%s",header,msg)
            self._flag = False
        cnt = 0
        self.recvmsg('/queue/test1',fun)
        self.send2queue('/queue/test1','test1233')
        while (self._flag == True and cnt < 5):
            cnt = cnt + 1
            time.sleep(1)
        if (self._flag == False):
            return 1,master
        else:
            return 0,''

    def __del__(self):
        if (self.client):
            self.client.disconnect()
    
class AMQCluster(object):
    def __init__(self,zk,ips,dev_user,dev_pass,adm_user,adm_pass):
        self._zk = zk
        self._ips = ips
        self._dev_user = dev_user
        self._dev_pass = dev_pass
        self._adm_user = adm_user
        self._adm_pass = adm_pass
        self._flag = True
    def get_master(self):
        zkobj = ZKUtil(self._zk)
        t = zkobj.children('/activemq/leveldb-stores')
        for k in t:
            m = zkobj.get('/activemq/leveldb-stores/' + k)
            dt = json.loads(m[0].decode())
            logging.debug("address:{},elected:{}".format(dt["address"],dt["elected"]))
            if (dt["address"] != None):
                logging.info(dt["address"])
                for ip in self._ips:
                    if (dt["address"].find("tcp://{}:".format(ip)) >= 0):
                        return ip
        return None
    def check(self):
        master = self.get_master()
        if (master == None):
            logging.info("not found master,zk:%s,ips:%s",self._zk,self._ips)
            return
        obj = AMQUtil(master,61613,self._dev_user,self._dev_pass)
        return obj.check(master)


if __name__ == '__main__':
    # obj = AMQUtil('10.130.9.123',61613,'XXXX','XXXXX')
    # def fun(header,msg):
    #     print(header)
    #     print(msg)    
    # obj.recvmsg('/queue/test1',fun)
    # while True:
    #     obj.send2queue('/queue/test1','test1233')
    #     time.sleep(1)
    pass





#coding=utf-8

from functools import wraps
from concurrent.futures import ThreadPoolExecutor
from concurrent.futures import ProcessPoolExecutor
class coming():
    def __init__(self,future,timeout):
        self.__future = future
        self.__timeout = timeout

    def __getattr__(self,name):
        ret = self.__wait()
        return ret.__getattribute__(name)

    def __wait(self):
        return self.__future.result(self.__timeout)
    
def async(n, base_type, timeout=None):
    def decorator(f):
        if isinstance(n, int):
            pool = base_type(n)
        elif isinstance(n, base_type):
            pool = n
        else:
            raise TypeError(
                "Invalid type: %s"
                % type(base_type)
            )
        @wraps(f)
        def wrapped(*args, **kwargs):
            return coming(
                pool.submit(f, *args, **kwargs),  # 创建future对象
                timeout=timeout
            )
        return wrapped
    return decorator


def threads(n, timeout=None):
    return async(n, ThreadPoolExecutor, timeout)


# import time
# import requests

# @threads(5,2)
# def download(url):
#     return requests.get(url)
# def download2(url):
#     return requests.get(url)

# if __name__ == "__main__":
#     urls = ['http://www.baidu.com','http://www.sohu.com','http://www.qq.com']
#     start = time.time()
#     responses = [download(url) for url in urls]
#     html = [response.text for response in responses]
#     end = time.time()
#     print "Time: %f seconds" % (end - start)


from pptx import Presentation
from pptx.chart.data import CategoryChartData
from pptx.enum.chart import XL_CHART_TYPE
from pptx.util import Inches
from pptx.util import Pt
from pptx.dml.color import RGBColor
from pptx.enum.chart import XL_TICK_MARK
from pptx.enum.shapes import MSO_SHAPE
from pptx.util import Cm

class PPTUtil(object):
    def __init__(self,ppt_temp):
        self.prs = Presentation(ppt_temp)

    '''
        新增报表幻灯片
        title ：幻灯片标题
        x_lab: 报表x轴
        x_val: 报表y轴
        slide_idx：依赖哪个幻灯片模版
        title_dix: 标题在模版中的位置
    '''
    def new_chart_slide(self,title,x_lab,x_val,slide_idx = 3,title_idx = 0):
        slide = self.prs.slides.add_slide(self.prs.slide_layouts[slide_idx])
        placeholder = slide.placeholders[title_idx]
        placeholder.text = title
        chart_data = CategoryChartData()
        chart_data.categories = x_lab
        chart_data.add_series('',x_val)

        x, y, cx, cy = Inches(0.5), Inches(2), Inches(12), Inches(3.5)
        graphic_frame = slide.shapes.add_chart(
            XL_CHART_TYPE.COLUMN_CLUSTERED, x, y, cx, cy, chart_data
        )
        chart = graphic_frame.chart
        category_axis = chart.category_axis
        category_axis.tick_labels.font.size = Pt(6)
        category_axis.has_major_gridlines = False
        category_axis.minor_tick_mark = XL_TICK_MARK.OUTSIDE

        value_axis = chart.value_axis
        tick_labels = value_axis.tick_labels
        tick_labels.font.size = Pt(8) 
        value_axis.has_major_gridlines = False
        value_axis.minor_tick_mark = XL_TICK_MARK.OUTSIDE

        bar_plot = chart.plots[0]
        bar_plot.gap_width = 20

        for i in range(len(x_lab)):
            point = chart.series[0].points[i]  # a (data) point in a bar chart is a bar            
            fill = point.format.fill
            fill.solid()
            fill.fore_color.rgb = RGBColor(102,171,163)
        tot = 0
        for v in x_val:
            tot = tot + v

        shape = slide.shapes.add_shape(MSO_SHAPE.RECTANGLE, Inches(4), Inches(6), Inches(6), Inches(1))
        text_frame = shape.text_frame
        text_frame.clear()  # not necessary for newly-created shape
        p = text_frame.paragraphs[0]
        run = p.add_run()
        run.text = str(tot)
        font = run.font
        font.name = 'Calibri'
        font.size = Pt(70)
        font.bold = True
    
    def new_tab_slide(self,title,tab_head,tab_data,slide_idx = 3,title_idx = 0,col_width = None):
        slide = self.prs.slides.add_slide(self.prs.slide_layouts[slide_idx])
        placeholder = slide.placeholders[title_idx]
        placeholder.text = title
        top    = Cm(3.5)
        left   = Cm(0.5)
        width  = Cm(28)
        height = Cm(10)

        table = slide.shapes.add_table(len(tab_data) + 1, len(tab_head), left, top, width, height).table
        for i in range(len(tab_head)):
            if (col_width == None or col_width[i] == None):
                table.columns[i].width = Cm(28/len(tab_head))
            else:
                table.columns[i].width = Cm(col_width[i])
        if (col_width == None):
            table.columns[len(tab_head) - 1].width = Cm(10)

        for i,v in enumerate(tab_head):
            cell = table.cell(0,i)
            cell.text = v
            paragraph = cell.text_frame.paragraphs[0]
            paragraph.font.size = Pt(10)

        for i,v in enumerate(tab_data):
            for j,d in enumerate(v):
                cell = table.rows[i + 1].cells[j]
                paragraph = cell.text_frame.paragraphs[0]
                paragraph.font.size = Pt(10)
                if (j == 7 and float(d) > 0):
                    paragraph.font.color.rgb = RGBColor(255, 0, 0)
                elif (j == 7 and float(d) < 0):
                    paragraph.font.color.rgb = RGBColor(0, 255, 0)
                else:
                    paragraph.font.color.rgb = RGBColor(12, 34, 56)
                cell.text = str(d)        


    def save(self,out_path,end_slide = None):
        if (end_slide != None):
            self.prs.slides.add_slide(self.prs.slide_layouts[end_slide])
        self.prs.save(out_path)


from elasticsearch import Elasticsearch

class ESUtil(object):
    def __init__(self,url,timeout = 200):
        self.es_url = url
        self.timeout = timeout
        self.es = Elasticsearch(self.es_url,timeout = self.timeout)
    
    '''
        从elasticsearch中聚合nginx日志
        默认是最近7天的，间隔3h聚合数据
    '''
    def aggs_nginx(self,msg_filter,dt_range = "7d",interval = "3h",index_name = 'logstash-*'):
        data = {
            "stored_fields": ["@timestamp","request"], 
            "query" : {
                "bool": {
                    "must" : {
                        "match":{"message":msg_filter}
                    },
                    "filter": {
                        "range":{
                            "@timestamp":{"gt":"now-" + dt_range}
                        }
                    }
                }
            },
            "aggs" : {
                "histo1" : {
                    "date_histogram" : {
                        "field" : "@timestamp",
                        "interval" : interval,
                        "time_zone":"+08:00"
                    }, 
                    "aggs": { 
                        "by_requests": { 
                            "terms":{
                                "field" : "@timestamp"
                            }
                        }
                    }
                }
            }
        }
        res = self.es.search(
            index = index_name,
            body = data
        )
        x_lab = []
        x_val = []
        for hit in res['aggregations']['histo1']['buckets']:
            x_lab.append(hit['key_as_string'].replace('T',' ').replace('+08:00',''))
            x_val.append(hit['doc_count'])
        return x_lab,x_val